Imports Microsoft.Xna.Framework

Namespace BigPicture
	Public Structure HorizontalCoordinate
		Public Sub New(ByVal azimuth As Double, ByVal altitude As Double, ByVal tilt As Double)
            Me.Azimuth = azimuth
			Me.Altitude = altitude
			Me.Tilt = tilt
		End Sub

		Public Sub New(ByVal azimuth As Double, ByVal altitude As Double)
			Me.New(azimuth, altitude, 0)
		End Sub

		' Eastward from north
        Private _azimuth As Double
        Public Property Azimuth As Double
            Get
                Return _azimuth
            End Get
            Private Set(ByVal value As Double)
                _azimuth = value
            End Set
        End Property

        Private _altitude As Double
        Public Property Altitude As Double
            Get
                Return _altitude
            End Get
            Private Set(ByVal value As Double)
                _altitude = value
            End Set
        End Property

        Private _tilt As Double
        Public Property Tilt As Double
            Get
                Return _tilt
            End Get
            Private Set(ByVal value As Double)
                _tilt = value
            End Set
        End Property

		Public Shared Function FromVector(ByVal vector As Vector3) As HorizontalCoordinate
			Dim altitude As Double = 180 * Math.Asin(vector.Z) / Math.PI

			' Make X and Y components negative for zero azimuth at south
			'  with westward increasing values.
			' Then change x and y calculations likewise in ToVector method
			Dim azimuth As Double = 180 * Math.Atan2(vector.X, vector.Y) / Math.PI

			Return New HorizontalCoordinate(azimuth, altitude)
		End Function

		Public Shared Function FromMotionMatrix(ByVal matrix As Matrix) As HorizontalCoordinate
			' Invert the matrix 
			matrix = Matrix.Invert(matrix)

			' Transform (0, 0, -1) -- the vector extending from the lends
			Dim zAxisTransformed As Vector3 = Vector3.Transform(-Vector3.UnitZ, matrix)

			' Get the horizontal coordinates
			Dim horzCoord As HorizontalCoordinate = FromVector(zAxisTransformed)

			' Find the theoretical HorizontalCoordinate for the transformed +Y vector if the phone is upright
			Dim yUprightAltitude As Double = 0
			Dim yUprightAzimuth As Double = 0

			If horzCoord.Altitude > 0 Then
				yUprightAltitude = 90 - horzCoord.Altitude
				yUprightAzimuth = 180 + horzCoord.Azimuth
			Else
				yUprightAltitude = 90 + horzCoord.Altitude
				yUprightAzimuth = horzCoord.Azimuth
			End If
			Dim yUprightVector As Vector3 = New HorizontalCoordinate(yUprightAzimuth, yUprightAltitude).ToVector

			' Find the real transformed +Y vector
			Dim yAxisTransformed As Vector3 = Vector3.Transform(Vector3.UnitY, matrix)

			' Get the angle between the upright +Y vector and the real transformed +Y vector
			Dim dotProduct As Double = Vector3.Dot(yUprightVector, yAxisTransformed)
			Dim crossProduct As Vector3 = Vector3.Cross(yUprightVector, yAxisTransformed)
			crossProduct.Normalize()

			Dim angleBetween As Double = 180 * Vector3.Dot(zAxisTransformed, crossProduct) * Math.Acos(dotProduct) / Math.PI
			horzCoord.Tilt = angleBetween

			Return horzCoord
		End Function

		Public Function ToVector() As Vector3
			Dim x As Double = Math.Cos(Math.PI * Me.Altitude / 180) * Math.Sin(Math.PI * Me.Azimuth / 180)
			Dim y As Double = Math.Cos(Math.PI * Me.Altitude / 180) * Math.Cos(Math.PI * Me.Azimuth / 180)
			Dim z As Double = Math.Sin(Math.PI * Me.Altitude / 180)

			Return New Vector3(CSng(x), CSng(y), CSng(z))
		End Function

		Public Overrides Function ToString() As String
			Return String.Format("Azi: {0} Alt: {1} Tilt: {2}", Me.Azimuth, Me.Altitude, Me.Tilt)
		End Function
	End Structure
End Namespace
