Imports System.ComponentModel
Imports System.IO
Imports System.Xml.Serialization
Imports System.Windows.Media.Imaging
Imports System.IO.IsolatedStorage

Namespace BigPicture
	Public Class PictureInfo
		Implements INotifyPropertyChanged

		Public Const ThumbnailsDirectory As String = "/thumbnails"

        Private _isDownloaded, _isDownloading As Boolean
        Private _downloadProgress As Double
        Private _downloadStatus As String

        Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

        <XmlAttribute()>
        Public Property Filename As String

        Public Property Url As String

        Public Property Title As String

        Public Property Artist As String

        Public ReadOnly Property Thumbnail As BitmapImage
            Get
                Dim bitmapImage As New BitmapImage
                Using isoStorage As IsolatedStorageFile = IsolatedStorageFile.GetUserStoreForApplication
                    Dim thumbnailFilename As String = ThumbnailsDirectory & "/" & Me.Filename

                    If isoStorage.FileExists(thumbnailFilename) Then
                        Using stream As IsolatedStorageFileStream = isoStorage.OpenFile(thumbnailFilename, FileMode.Open, FileAccess.Read)
                            bitmapImage.SetSource(stream)
                        End Using
                    End If
                End Using
                Return bitmapImage
            End Get
        End Property

        Public Property IsDownloaded As Boolean
            Set(ByVal value As Boolean)
                If _isDownloaded <> value Then
                    _isDownloaded = value
                    OnPropertyChanged("IsDownloaded")
                    OnPropertyChanged("IsNotDownloaded")
                    OnPropertyChanged("Thumbnail")
                End If
            End Set
            Get
                Return _isDownloaded
            End Get
        End Property

        Public ReadOnly Property IsNotDownloaded As Boolean
            Get
                Return Not _isDownloaded
            End Get
        End Property

        Public Property IsDownloading As Boolean
            Set(ByVal value As Boolean)
                If _isDownloading <> value Then
                    _isDownloading = value
                    OnPropertyChanged("IsDownloading")
                    OnPropertyChanged("IsNotDownloading")
                End If
            End Set
            Get
                Return _isDownloading
            End Get
        End Property

        Public ReadOnly Property IsNotDownloading As Boolean
            Get
                Return Not _isDownloading
            End Get
        End Property

        Public Property DownloadProgress As Double
            Set(ByVal value As Double)
                If _downloadProgress <> value Then
                    _downloadProgress = value
                    OnPropertyChanged("DownloadProgress")
                End If
            End Set
            Get
                Return _downloadProgress
            End Get
        End Property

        Public Property DownloadRequestId As String

        Public Property DownloadStatus As String
            Set(ByVal value As String)
                If _downloadStatus <> value Then
                    _downloadStatus = value
                    OnPropertyChanged("DownloadStatus")
                End If
            End Set
            Get
                Return _downloadStatus
            End Get
        End Property

		Protected Overridable Sub OnPropertyChanged(ByVal propertyName As String)
			RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
		End Sub
	End Class
End Namespace
